<?php
// src/Controller/CategoriesController.php
namespace App\Controller;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Serializer\Encoder\JsonEncoder;
use Symfony\Component\Serializer\Normalizer\GetSetMethodNormalizer;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use App\Entity\Categoria;

class CategoriesController extends AbstractController
{
	/**
     * @Route("/nova-categoria", name="nova_categoria")
     */
    public function newCategory(Request $request)
    {
    	if(isset($request->request)) {
    		$session = $request->getSession();
        	$usuari_logged = $session->get('usuari_logged');

        	$data = array(
	        	'usuari_logged' => $usuari_logged,
	        	'categories' => array()
	        );

        	if (!empty($usuari_logged) && $usuari_logged->getId() != NULL) {

        		$repository1 = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
				$categories = $repository1->findBy(
				    ['estat' => 'APROVAT'],
				    ['nom' => 'ASC']
				);
				$categories_prod = array();
				foreach ($categories as $categoria) {
		            if (sizeof($categoria->getItems()) > 0) {
		            	array_push($categories_prod, $categoria);
		            }
		        }
		        $data['categories'] = $categories_prod;

        		return $this->render('/nova-categoria.html.twig', $data);
	        }
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/addCategoria", name="add-categoria")
     */
    public function addCategoria(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$nom = $request->request->get('nom');
            $descripcio = $request->request->get('descripcio');
            $motiu = $request->request->get('motiu');

    		if (!empty($nom) && !empty($descripcio) && !empty($motiu)) {
				
				$session = $request->getSession();
        		$usuari_logged = $session->get('usuari_logged');

        		if (!empty($usuari_logged)) {
        			
        			$repository1 = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
					$usuaris = $repository1->findBy(
				    	['id' => $usuari_logged->getId()]
					);
					if (!empty($usuaris)) {
	        			$usuari = current($usuaris);
	        			$categoria = new Categoria();
		                $categoria->setNom($nom);
		                $categoria->setDescripcio($descripcio);
		                $categoria->setMotiuCreacio($motiu);
		                $categoria->setEstat('APROVAT');
		                $date = new \DateTime();
		                $categoria->setDataCreacio($date);
		                $categoria->setUsuari($usuari);

		                $entityManager = $this->getDoctrine()->getManager();
		                $entityManager->persist($categoria);
		                $entityManager->flush();

		                return new JsonResponse(json_encode(array('success' => $categoria->getNomUrl())));
		            }
        		}

        		return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
			}
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/loadMoreCategories", name="load-more-categories")
     */
    public function loadMoreCategories(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$indx = $request->request->get('indx');

    		if (!empty($indx)) {
				
				$session = $request->getSession();
        		$usuari_logged = $session->get('usuari_logged');

        		$repository1 = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
        		$categories = $repository1->findBy(
				    ['estat' => 'APROVAT'],
				    ['nom' => 'ASC']
				);
				$cont = 0;
				$categories_prod = array();
				foreach ($categories as $categoria) {
		            if (sizeof($categoria->getItems()) > 0 && $cont > $indx) {
		            	array_push($categories_prod, $categoria);
		            }
		            $cont = $cont + 1;
		        }

		        $normalizer = new GetSetMethodNormalizer();
		        $normalizer->setIgnoredAttributes(array("usuari", "items"));
				$encoder = new JsonEncoder();

				$serializer = new Serializer(array($normalizer), array($encoder));
				$jsonContent = $serializer->serialize($categories_prod, 'json', [
	    			'circular_reference_handler' => function ($object) {
	        			return $object->getId();
	        		}
	    		]);

        		return new JsonResponse($jsonContent);
			}

			return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
		}

		return $this->redirectToRoute('index');
    }
}