<?php
// src/Controller/ItemsController.php
namespace App\Controller;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Serializer\Encoder\JsonEncoder;
use Symfony\Component\Serializer\Normalizer\GetSetMethodNormalizer;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use App\Entity\Item;
use App\Entity\Comparacio;

class ItemsController extends AbstractController
{
	/**
     * @Route("/getItemsByCategory", name="items_by_category")
     */
    public function getItemsByCategory(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$categoria_id = $request->request->get('categoria_id');

    		$repository = $this->getDoctrine()->getRepository(\App\Entity\Item::class);
			$items = $repository->findBy(
		    	['categoria' => $categoria_id],
		    	['puntuacio' => 'DESC']
			);

			$normalizer = new GetSetMethodNormalizer();
			$normalizer->setIgnoredAttributes(array('contrasenya'));
			$encoder = new JsonEncoder();

			$serializer = new Serializer(array($normalizer), array($encoder));
			$jsonContent = $serializer->serialize($items, 'json', [
    			'circular_reference_handler' => function ($object) {
        			return $object->getId();
        		}
    		]);

        	return new JsonResponse($jsonContent);
		}

		return new JsonResponse();
    }

    /**
     * @Route("/item/{slug}", name="load-item")
     */
    public function loadItem(Request $request, $slug)
    {
    	if(isset($request->request)) {
    		$item_id = $slug;

    		$repository = $this->getDoctrine()->getRepository(\App\Entity\Item::class);
			$items = $repository->findBy(
		    	['id' => $item_id]
			);

			$item = current($items);
			if (!empty($item)) {
				$session = $request->getSession();
        		$usuari_logged = $session->get('usuari_logged');

				$data = array(
					'item' => $item,
					'usuari_logged' => $usuari_logged,
					'items_relacionats' => array()
				);

        		$categoria = $item->getCategoria();
        		if (!empty($categoria)) {
        			$items1 = $categoria->getItems();
        			if (!empty($items1)) {
        				$items_prod = array();
        				$i = 0;
        				foreach ($items1 as $it) {
				            if (intval($it->getId()) != intval($item_id)) {
				            	array_push($data['items_relacionats'], $it);
				            }
				            $i = $i + 1;
				            if ($i > 3) {
				            	break;
				            }
				        }
        			}
        		}

        		return $this->render('/item.html.twig', $data);
			}
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/updateItemDescription", name="update-item-description")
     */
    public function updateItemDescription(Request $request)
    {
        if(isset($request->request) && $request->isXmlHttpRequest()) {
            $item_id = $request->request->get('id_item');
            $descripcio = $request->request->get('descripcio');

            $entityManager = $this->getDoctrine()->getManager();
            $repository = $this->getDoctrine()->getRepository(\App\Entity\Item::class);
            $items = $repository->findBy(
                ['id' => intval($item_id)]
            );
            $item = current($items);

            if (!empty($item)) {
                $item->setDescripcio($descripcio);

                $entityManager->persist($item);
                $entityManager->flush();
                
                return new JsonResponse(json_encode(array('success' => 'OK')));
                
            }
        }

        return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

    /**
     * @Route("/updateItemAdvDis", name="update-item-adv-dis")
     */
    public function updateItemAdvDis(Request $request)
    {
        if(isset($request->request) && $request->isXmlHttpRequest()) {
            $item_id = $request->request->get('id_item');
            $avantatges = $request->request->get('avantatges');
            $inconvenients = $request->request->get('inconvenients');

            if ((!empty($avantatges)) || (!empty($inconvenients))) {
	            $entityManager = $this->getDoctrine()->getManager();
	            $repository = $this->getDoctrine()->getRepository(\App\Entity\Item::class);
	            $items = $repository->findBy(
	                ['id' => intval($item_id)]
	            );
	            $item = current($items);

	            if (!empty($item)) {
	                if (!empty($avantatges)) {
	                	$item->setAvantatges($avantatges);
	                }
	                if (!empty($inconvenients)) {
	                	$item->setInconvenients($inconvenients);
	                }

	                $entityManager->persist($item);
	                $entityManager->flush();
	                
	                return new JsonResponse(json_encode(array('success' => 'OK')));
	                
	            }
	        }
        }

        return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

    /**
     * @Route("/nou-item", name="nou-item")
     */
    public function nouItem(Request $request)
    {
    	if(isset($request->request)) {
    		$session = $request->getSession();
        	$usuari_logged = $session->get('usuari_logged');

        	$repository1 = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
			$categories = $repository1->findBy(
			    ['estat' => 'APROVAT'],
			    ['nom' => 'ASC']
			);

        	$data = array(
	        	'usuari_logged' => $usuari_logged,
	        	'categories' => $categories,
	        	'items_usuari' => array()
	        );

        	if (!empty($usuari_logged) && $usuari_logged->getId() != NULL) {

        		$repository = $this->getDoctrine()->getRepository(\App\Entity\Item::class);
				$items = $repository->findBy(
			    	['usuari' => $usuari_logged->getId()],
			    	['puntuacio' => 'DESC'],
			    	3
				);
				if (!empty($items)) {
					$data['items_usuari'] = $items;
				}

	    		return $this->render('/nou-item.html.twig', $data);
	        }
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/items/{slug}", name="load-items")
     */
    public function loadItems(Request $request, $slug)
    {
    	if(isset($request->request)) {
    		$item_category = $slug;

    		$repository = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
			$categories = $repository->findBy(
		    	['nom' => $item_category]
			);

			$categoria = current($categories);
			if (!empty($categoria)) {
				$session = $request->getSession();
        		$usuari_logged = $session->get('usuari_logged');

        		$categories1 = $repository->findBy(
				    ['estat' => 'APROVAT'],
				    ['nom' => 'ASC']
				);
				$categories_prod = array();
				foreach ($categories1 as $cat) {
		            if (sizeof($cat->getItems()) > 0) {
		            	array_push($categories_prod, $cat);
		            }
		        }

        		return $this->render('/items.html.twig', array(
                	'categoria' => $categoria,
                	'categories' => $categories_prod,
                	'usuari_logged' => $usuari_logged
            	));
			}
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/addItem", name="add-item")
     */
    public function addItem(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$nom = $request->request->get('nom');
            $descripcio = $request->request->get('descripcio');
            $categoria_id = $request->request->get('categoria');
            $puntuacio = $request->request->get('puntuacio');
            $avantatges = $request->request->get('avantatges');
            $inconvenients = $request->request->get('inconvenients');
            $imatge = $request->files->get('imatge');

    		if (!empty($nom) && !empty($descripcio) && !empty($categoria_id) && 
    			$puntuacio >= 0) {
				
				$session = $request->getSession();
        		$usuari_logged = $session->get('usuari_logged');

        		if (!empty($usuari_logged)) {
        			
        			$repository = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
					$categories = $repository->findBy(
				    	['id' => $categoria_id]
					);
					$repository1 = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
					$usuaris = $repository1->findBy(
				    	['id' => $usuari_logged->getId()]
					);
					if (!empty($categories) && !empty($usuaris)) {
	        			$categoria = current($categories);
	        			$usuari = current($usuaris);
	        			$item = new Item();
		                $item->setNom($nom);
		                $item->setDescripcio($descripcio);
		                $item->setCategoria($categoria);
		                $item->setPuntuacio(intval($puntuacio)*2);
		                $item->setAvantatges($avantatges);
		                $item->setInconvenients($inconvenients);
		                $date = new \DateTime();
		                $item->setDataCreacio($date);
		                $item->setUsuari($usuari);

		                if (!empty($imatge)) {
		                	$fileName = md5(uniqid()).'.'.$imatge->guessExtension();
		                	//try {
				                $publicDir = $this->getParameter('kernel.project_dir') . '/public/'; 
				                $imatge->move(
				                    $publicDir.'/img/front/',
				                    $fileName
				                );
				            //} catch (FileException $e) {}
		                	$item->setImatge($fileName);
		                }

		                $entityManager = $this->getDoctrine()->getManager();
		                $entityManager->persist($item);
		                $entityManager->flush();

		                return new JsonResponse(json_encode(array('success' => $item->getId())));
		            }
        		}

        		return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
			}
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/cerca", name="search-items")
     */
    public function searchItems(Request $request)
    {
    	if(isset($request->request)) {
    		$terme = $request->query->get('terme');

    		if (!empty($terme)) {
    			$session = $request->getSession();
        		$usuari_logged = $session->get('usuari_logged');

        		$entityManager = $this->getDoctrine()->getManager();
        		$result = $entityManager->getRepository(\App\Entity\Item::class)->createQueryBuilder('item')
					->where('item.nom LIKE :terme or item.descripcio LIKE :terme')
				    ->setParameter('terme', '%'.$terme.'%')
				    ->getQuery()
				    ->getResult();

				$repository2 = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
				$categories = $repository2->findBy(
				    ['estat' => 'APROVAT'],
				    ['nom' => 'ASC']
				);
				$categories_prod = array();
				foreach ($categories as $cat) {
		            if (sizeof($cat->getItems()) > 0) {
		            	array_push($categories_prod, $cat);
		            }
		        }

				$data = array(
					'usuari_logged' => $usuari_logged,
					'resultats' => $result,
					'categories' => $categories_prod,
					'terme' => $terme
				);

				return $this->render('/resultats.html.twig', $data);
    		}
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/loadMoreItems", name="load-more-items")
     */
    public function loadMoreItems(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$indx = $request->request->get('indx');
    		$categoria_id = $request->request->get('categoria');

    		if (!empty($indx)) {
				
				$repository1 = $this->getDoctrine()->getRepository(\App\Entity\Item::class);
        		$items = $repository1->findBy(
				    ['categoria' => intval($categoria_id)]
				);
				$cont = 0;
				$items_prod = array();
				foreach ($items as $item) {
		            if ($cont >= $indx) {
		            	array_push($items_prod, $item);
		            }
		            $cont = $cont + 1;
		            if ($cont >= (10 + $indx)) {
		            	break;
		            }
		        }

		        $normalizer = new GetSetMethodNormalizer();
		        $normalizer->setIgnoredAttributes(array("contrasenya","categoria","usuari","comparacions"));
				$encoder = new JsonEncoder();

				$serializer = new Serializer(array($normalizer), array($encoder));
				$jsonContent = $serializer->serialize($items_prod, 'json', [
	    			'circular_reference_handler' => function ($object) {
	        			return $object->getId();
	        		}
	    		]);

        		return new JsonResponse($jsonContent);
			}

			return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
		}

		return $this->redirectToRoute('index');
    }

    /**
     * @Route("/loadMoreResultats", name="load-more-resultats")
     */
    public function loadMoreResultats(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$indx = $request->request->get('indx');
    		$terme = $request->request->get('terme');

    		if (!empty($indx) && !empty($terme)) {
				
				$entityManager = $this->getDoctrine()->getManager();
        		$items = $entityManager->getRepository(\App\Entity\Item::class)->createQueryBuilder('item')
					->where('item.nom LIKE :terme or item.descripcio LIKE :terme')
				    ->setParameter('terme', '%'.$terme.'%')
				    ->getQuery()
				    ->getResult();
				
				$cont = 0;
				$items_prod = array();
				foreach ($items as $item) {
		            if ($cont >= $indx) {
		            	array_push($items_prod, $item);
		            }
		            $cont = $cont + 1;
		            if ($cont >= (10 + $indx)) {
		            	break;
		            }
		        }

		        $normalizer = new GetSetMethodNormalizer();
		        $normalizer->setIgnoredAttributes(array("contrasenya","categoria","usuari","comparacions"));
				$encoder = new JsonEncoder();

				$serializer = new Serializer(array($normalizer), array($encoder));
				$jsonContent = $serializer->serialize($items_prod, 'json', [
	    			'circular_reference_handler' => function ($object) {
	        			return $object->getId();
	        		}
	    		]);

        		return new JsonResponse($jsonContent);
			}

			return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
		}

		return $this->redirectToRoute('index');
    }
}