<?php
// src/Controller/UsuarissController.php
namespace App\Controller;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Serializer\Encoder\JsonEncoder;
use Symfony\Component\Serializer\Normalizer\GetSetMethodNormalizer;
use Symfony\Component\Serializer\Normalizer\ObjectNormalizer;
use Symfony\Component\Serializer\Serializer;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Session\Session;
use App\Entity\Usuari;
use App\Entity\Valoracio;

class UsuarisController extends AbstractController
{

    /**
     * @Route("/usuari/{slug}", name="default")
     */
    public function default(Request $request, $slug)
    {
        $repository = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
        $usuaris = $repository->findBy(
            ['id' => $slug]
        );

        $session = $request->getSession();
        $usuari_logged = $session->get('usuari_logged');

        if (!empty($usuaris)) {
            $usuari = current($usuaris);

            $repository1 = $this->getDoctrine()->getRepository(\App\Entity\Categoria::class);
            $categories = $repository1->findBy(
                ['estat' => 'APROVAT'],
                ['nom' => 'ASC']
            );
            $categories_prod = array();
            foreach ($categories as $cat) {
                if (sizeof($cat->getItems()) > 0) {
                    array_push($categories_prod, $cat);
                }
            }

            return $this->render('/usuari.html.twig', array(
                'usuari_info' => $usuari,
                'usuari_logged' => $usuari_logged,
                'categories' => $categories_prod
            ));
        } else {
            return $this->redirectToRoute('index');
        }

        return $this->render('/usuari.html.twig', array(
            'usuari_info' => NULL,
            'usuari_logged' => $usuari_logged
        ));
    }

	/**
     * @Route("/doLogin", name="login")
     */
    public function doLogin(Request $request)
    {
    	if(isset($request->request) && $request->isXmlHttpRequest()) {
    		$email = $request->request->get('email');
            $contrasenya = $request->request->get('contrasenya');

    		$repository = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
			$usuaris = $repository->findBy(
		    	['email' => $email]
			);
            if (!empty($usuaris)) {
                $usuari = current($usuaris);
                $hashed_password = password_hash($contrasenya, PASSWORD_DEFAULT);
                if (password_verify($contrasenya, $usuari->getContrasenya())) {
        			$normalizer = new GetSetMethodNormalizer();
        			$normalizer->setIgnoredAttributes(array('contrasenya','items','comparacions','categories'));
        			$encoder = new JsonEncoder();

        			$serializer = new Serializer(array($normalizer), array($encoder));
        			$jsonContent = $serializer->serialize(current($usuaris), 'json', [
            			'circular_reference_handler' => function ($object) {
                			return $object->getId();
                		}
            		]);

                    $session = $request->getSession();
                    $session->set('usuari_logged', $usuari);

                	return new JsonResponse($jsonContent);
                } else {
                    return new JsonResponse(json_encode(array('info' => 'La contrasenya introduïda no és vàlida')));
                }
            } else {
                return new JsonResponse(json_encode(array('info' => 'El correu introduït no existeix')));
            }
		}

		return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

    /**
     * @Route("/doSignUp", name="sign-up")
     */
    public function doSignUp(Request $request)
    {
        if(isset($request->request) && $request->isXmlHttpRequest()) {
            $nom = $request->request->get('nom');
            $email = $request->request->get('email');
            $contrasenya = $request->request->get('contrasenya');

            $entityManager = $this->getDoctrine()->getManager();
            $repository = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
            $usuaris = $repository->findBy(
                ['email' => $email]
            );

            if (empty($usuaris)) {
                $hashed_password = password_hash($contrasenya, PASSWORD_DEFAULT);

                $usuari = new Usuari();
                $usuari->setNom($nom);
                $usuari->setLlinatges('');
                $usuari->setEmail($email);
                $usuari->setContrasenya($hashed_password);
                $usuari->setDescripcio('');
                $usuari->setPuntuacio(0);
                $date = new \DateTime();
                $usuari->setDataCreacio($date);

                $entityManager->persist($usuari);
                $entityManager->flush();
                
                $normalizer = new GetSetMethodNormalizer();
                $normalizer->setIgnoredAttributes(array('contrasenya'));
                $encoder = new JsonEncoder();

                $serializer = new Serializer(array($normalizer), array($encoder));
                $jsonContent = $serializer->serialize($usuari, 'json', [
                    'circular_reference_handler' => function ($object) {
                        return $object->getId();
                    }
                ]);

                $session = $request->getSession();
                $session->set('usuari_logged', $usuari);

                return new JsonResponse($jsonContent);
                
            } else {
                return new JsonResponse(json_encode(array('info' => 'Ja existeix un usuari amb aquest correu')));
            }
        }

        return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

    /**
     * @Route("/doLogout", name="logout")
     */
    public function doLogout(Request $request)
    {
        if(isset($request->request) && $request->isXmlHttpRequest()) {
            $session = $request->getSession();
            $session->set('usuari_logged', NULL);

            return new JsonResponse(json_encode(array('success' => 'OK')));
        }

        return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

    /**
     * @Route("/postComment", name="post-comment")
     */
    public function postComment(Request $request)
    {
        if(isset($request->request) && $request->isXmlHttpRequest()) {
            $usuari_origen_id = $request->request->get('id_ori');
            $usuari_desti_id = $request->request->get('id_dst');
            $puntuacio = $request->request->get('puntuacio');
            $comentari = $request->request->get('valoracio');

            $entityManager = $this->getDoctrine()->getManager();
            $repository = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
            $usuaris1 = $repository->findBy(
                ['id' => intval($usuari_origen_id)]
            );
            $usuaris2 = $repository->findBy(
                ['id' => intval($usuari_desti_id)]
            );
            $usuari_origen = current($usuaris1);
            $usuari_desti = current($usuaris2);

            if (!empty($usuari_origen) && !empty($usuari_desti) && !empty($puntuacio) && !empty($comentari)) {
                $valoracio = new Valoracio();
                $valoracio->setUsuari($usuari_origen);
                $valoracio->setUsuariDesti($usuari_desti);
                $valoracio->setPuntuacio(intval($puntuacio)*2);
                $valoracio->setValor($comentari);
                $date = new \DateTime();
                $valoracio->setDataCreacio($date);

                $entityManager->persist($valoracio);
                $entityManager->flush();

                $nota_mitja = 0;
                $ind = 0;
                foreach ($usuari_desti->getValoracions() as $val) {
                    $ind = $ind + 1;
                    $nota = $val->getPuntuacio();
                    $nota_mitja = $nota_mitja + $nota;
                }
                if ($ind > 0) {
                    $nota_mitja = $nota_mitja / $ind;
                    $usuari_desti->setPuntuacio(intval(round($nota_mitja)));
                } else {
                    $usuari_desti->setPuntuacio(intval($puntuacio)*2);
                }
                $entityManager->persist($usuari_desti);
                $entityManager->flush();
                
                return new JsonResponse(json_encode(array('success' => 'OK')));
                
            } else {
                return new JsonResponse(json_encode(array('info' => 'Revisa els valors introduits')));
            }
        }

        return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

    /**
     * @Route("/updateDescription", name="update-description")
     */
    public function updateDescription(Request $request)
    {
        if(isset($request->request) && $request->isXmlHttpRequest()) {
            $usuari_id = $request->request->get('id_usuari');
            $descripcio = $request->request->get('descripcio');
            $imatge = $request->files->get('imatge');

            $entityManager = $this->getDoctrine()->getManager();
            $repository = $this->getDoctrine()->getRepository(\App\Entity\Usuari::class);
            $usuaris = $repository->findBy(
                ['id' => intval($usuari_id)]
            );
            $usuari = current($usuaris);

            if (!empty($usuari)) {
                $usuari->setDescripcio($descripcio);

                if (!empty($imatge)) {
                    $fileName = md5(uniqid()).'.'.$imatge->guessExtension();
                    //try {
                        $publicDir = $this->getParameter('kernel.project_dir') . '/public/'; 
                        $imatge->move(
                            $publicDir.'/img/front/usuaris/',
                            $fileName
                        );
                    //} catch (FileException $e) {}
                    $usuari->setImatge($fileName);
                }

                $entityManager->persist($usuari);
                $entityManager->flush();
                
                return new JsonResponse(json_encode(array('success' => 'OK')));
                
            }
        }

        return new JsonResponse(json_encode(array('info' => 'S\'ha produït un error, si us plau torna-ho a intentar en uns minuts')));
    }

}