<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity(repositoryClass="App\Repository\CategoriaRepository")
 */
class Categoria
{
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue()
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $nom;

    /**
     * @ORM\Column(type="text")
     */
    private $descripcio;

    /**
     * @ORM\Column(type="text")
     */
    private $motiu_creacio;

    /**
     * @ORM\Column(type="string", length=32)
     */
    private $estat;

    /**
     * @ORM\Column(type="datetime")
     */
    private $data_creacio;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Usuari", inversedBy="categorias")
     * @ORM\JoinColumn(nullable=false)
     */
    private $usuari;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Item", mappedBy="categoria")
     */
    private $items;

    public function __construct()
    {
        $this->items = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function getNomUrl(): ?string
    {
        return strtr(utf8_decode($this->nom), utf8_decode('àáâãäçèéêëìíîïñòóôõöùúûüýÿÀÁÂÃÄÇÈÉÊËÌÍÎÏÑÒÓÔÕÖÙÚÛÜÝ'), 'aaaaaceeeeiiiinooooouuuuyyAAAAACEEEEIIIINOOOOOUUUUY');
    }

    public function setNom(string $nom): self
    {
        $this->nom = $nom;

        return $this;
    }

    public function getDescripcio(): ?string
    {
        return $this->descripcio;
    }

    public function setDescripcio(string $descripcio): self
    {
        $this->descripcio = $descripcio;

        return $this;
    }

    public function getMotiuCreacio(): ?string
    {
        return $this->motiu_creacio;
    }

    public function setMotiuCreacio(string $motiu_creacio): self
    {
        $this->motiu_creacio = $motiu_creacio;

        return $this;
    }

    public function getEstat(): ?string
    {
        return $this->estat;
    }

    public function setEstat(string $estat): self
    {
        $this->estat = $estat;

        return $this;
    }

    public function getDataCreacio(): ?\DateTimeInterface
    {
        return $this->data_creacio;
    }

    public function setDataCreacio(\DateTimeInterface $data_creacio): self
    {
        $this->data_creacio = $data_creacio;

        return $this;
    }

    public function getUsuari(): ?Usuari
    {
        return $this->usuari;
    }

    public function setUsuari(?Usuari $usuari): self
    {
        $this->usuari = $usuari;

        return $this;
    }

    /**
     * @return Collection|Item[]
     */
    public function getItems(): Collection
    {
        return $this->items;
    }

    public function addItem(Item $item): self
    {
        if (!$this->items->contains($item)) {
            $this->items[] = $item;
            $item->setCategoria($this);
        }

        return $this;
    }

    public function removeItem(Item $item): self
    {
        if ($this->items->contains($item)) {
            $this->items->removeElement($item);
            // set the owning side to null (unless already changed)
            if ($item->getCategoria() === $this) {
                $item->setCategoria(null);
            }
        }

        return $this;
    }
}
