<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity(repositoryClass="App\Repository\ItemRepository")
 */
class Item
{
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue()
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=128)
     */
    private $nom;

    /**
     * @ORM\Column(type="text")
     */
    private $descripcio;

    /**
     * @ORM\Column(type="text")
     */
    private $avantatges;

    /**
     * @ORM\Column(type="text")
     */
    private $inconvenients;

    /**
     * @ORM\Column(type="integer")
     */
    private $puntuacio;

    /**
     * @ORM\Column(type="datetime")
     */
    private $data_creacio;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Categoria", inversedBy="items")
     * @ORM\JoinColumn(nullable=false)
     */
    private $categoria;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Usuari", inversedBy="items")
     * @ORM\JoinColumn(nullable=false)
     */
    private $usuari;

    /**
     * @ORM\ManyToMany(targetEntity="App\Entity\Comparacio", mappedBy="items")
     */
    private $comparacions;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Assert\NotBlank(message="Puja una imatge")
     * @Assert\File(mimeTypes={ "image/png", "image/jpeg" })
     */
    private $imatge;

    public function __construct()
    {
        $this->comparacions = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): self
    {
        $this->nom = $nom;

        return $this;
    }

    public function getDescripcio(): ?string
    {
        return $this->descripcio;
    }

    public function setDescripcio(string $descripcio): self
    {
        $this->descripcio = $descripcio;

        return $this;
    }

    public function getAvantatges(): ?string
    {
        return $this->avantatges;
    }

    public function setAvantatges(string $avantatges): self
    {
        $this->avantatges = $avantatges;

        return $this;
    }

    public function getInconvenients(): ?string
    {
        return $this->inconvenients;
    }

    public function setInconvenients(string $inconvenients): self
    {
        $this->inconvenients = $inconvenients;

        return $this;
    }

    public function getPuntuacio(): ?int
    {
        return $this->puntuacio;
    }

    public function setPuntuacio(int $puntuacio): self
    {
        $this->puntuacio = $puntuacio;

        return $this;
    }

    public function getDataCreacio(): ?\DateTimeInterface
    {
        return $this->data_creacio;
    }

    public function setDataCreacio(\DateTimeInterface $data_creacio): self
    {
        $this->data_creacio = $data_creacio;

        return $this;
    }

    public function getCategoria(): ?Categoria
    {
        return $this->categoria;
    }

    public function setCategoria(?Categoria $categoria): self
    {
        $this->categoria = $categoria;

        return $this;
    }

    public function getUsuari(): ?Usuari
    {
        return $this->usuari;
    }

    public function setUsuari(?Usuari $usuari): self
    {
        $this->usuari = $usuari;

        return $this;
    }

    /**
     * @return Collection|Comparacio[]
     */
    public function getComparacions(): Collection
    {
        return $this->comparacions;
    }

    public function addComparacion(Comparacio $comparacion): self
    {
        if (!$this->comparacions->contains($comparacion)) {
            $this->comparacions[] = $comparacion;
            $comparacion->addItem($this);
        }

        return $this;
    }

    public function removeComparacion(Comparacio $comparacion): self
    {
        if ($this->comparacions->contains($comparacion)) {
            $this->comparacions->removeElement($comparacion);
            $comparacion->removeItem($this);
        }

        return $this;
    }

    public function getImatge(): ?string
    {
        return $this->imatge;
    }

    public function setImatge(?string $imatge): self
    {
        $this->imatge = $imatge;

        return $this;
    }
}
