<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity(repositoryClass="App\Repository\UsuariRepository")
 */
class Usuari
{
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue()
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $email;

    /**
     * @ORM\Column(type="string", length=128)
     */
    private $nom;

    /**
     * @ORM\Column(type="string", length=128)
     */
    private $llinatges;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $contrasenya;

    /**
     * @ORM\Column(type="text")
     */
    private $descripcio;

    /**
     * @ORM\Column(type="integer")
     */
    private $puntuacio;

    /**
     * @ORM\Column(type="datetime")
     */
    private $data_creacio;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Categoria", mappedBy="usuari")
     */
    private $categorias;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Item", mappedBy="usuari")
     */
    private $items;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Comparacio", mappedBy="usuari")
     */
    private $comparacions;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Comentari", mappedBy="usuari")
     */
    private $comentaris;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Valoracio", mappedBy="usuari")
     */
    private $valoracions;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Valoracio", mappedBy="usuari_desti")
     */
    private $valoracions_from;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     * @Assert\NotBlank(message="Puja una imatge")
     * @Assert\File(mimeTypes={ "image/png", "image/jpeg" })
     */
    private $imatge;


    public function __construct()
    {
        $this->categorias = new ArrayCollection();
        $this->items = new ArrayCollection();
        $this->comentaris = new ArrayCollection();
        $this->comparacions = new ArrayCollection();
        $this->valoracions = new ArrayCollection();
        $this->valoracions_from = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(string $email): self
    {
        $this->email = $email;

        return $this;
    }

    public function getNom(): ?string
    {
        return $this->nom;
    }

    public function setNom(string $nom): self
    {
        $this->nom = $nom;

        return $this;
    }

    public function getLlinatges(): ?string
    {
        return $this->llinatges;
    }

    public function setLlinatges(string $llinatges): self
    {
        $this->llinatges = $llinatges;

        return $this;
    }

    public function getContrasenya(): ?string
    {
        return $this->contrasenya;
    }

    public function setContrasenya(string $contrasenya): self
    {
        $this->contrasenya = $contrasenya;

        return $this;
    }

    public function getDescripcio(): ?string
    {
        return $this->descripcio;
    }

    public function setDescripcio(string $descripcio): self
    {
        $this->descripcio = $descripcio;

        return $this;
    }

    public function getPuntuacio(): ?int
    {
        return $this->puntuacio;
    }

    public function setPuntuacio(int $puntuacio): self
    {
        $this->puntuacio = $puntuacio;

        return $this;
    }

    public function getDataCreacio(): ?\DateTimeInterface
    {
        return $this->data_creacio;
    }

    public function setDataCreacio(\DateTimeInterface $data_creacio): self
    {
        $this->data_creacio = $data_creacio;

        return $this;
    }

    /**
     * @return Collection|Categoria[]
     */
    public function getCategorias(): Collection
    {
        return $this->categorias;
    }

    public function addCategoria(Categoria $categoria): self
    {
        if (!$this->categorias->contains($categoria)) {
            $this->categorias[] = $categoria;
            $categoria->setUsuari($this);
        }

        return $this;
    }

    public function removeCategoria(Categoria $categoria): self
    {
        if ($this->categorias->contains($categoria)) {
            $this->categorias->removeElement($categoria);
            // set the owning side to null (unless already changed)
            if ($categoria->getUsuari() === $this) {
                $categoria->setUsuari(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Item[]
     */
    public function getItems(): Collection
    {
        return $this->items;
    }

    public function addItem(Item $item): self
    {
        if (!$this->items->contains($item)) {
            $this->items[] = $item;
            $item->setUsuari($this);
        }

        return $this;
    }

    public function removeItem(Item $item): self
    {
        if ($this->items->contains($item)) {
            $this->items->removeElement($item);
            // set the owning side to null (unless already changed)
            if ($item->getUsuari() === $this) {
                $item->setUsuari(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Comparacio[]
     */
    public function getComparacions(): Collection
    {
        return $this->comparacions;
    }

    public function addComparacion(Comparacio $comparacion): self
    {
        if (!$this->comparacions->contains($comparacion)) {
            $this->comparacions[] = $comparacion;
            $comparacion->setUsuari($this);
        }

        return $this;
    }

    public function removeComparacion(Comparacio $comparacion): self
    {
        if ($this->comparacions->contains($comparacion)) {
            $this->comparacions->removeElement($comparacion);
            // set the owning side to null (unless already changed)
            if ($comparacion->getUsuari() === $this) {
                $comparacion->setUsuari(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Comentari[]
     */
    public function getComentaris(): Collection
    {
        return $this->comentaris;
    }

    public function addComentari(Comentari $comentari): self
    {
        if (!$this->comentaris->contains($comentari)) {
            $this->comentaris[] = $comentari;
            $comentari->setUsuari($this);
        }

        return $this;
    }

    public function removeComentari(Comentari $comentari): self
    {
        if ($this->comentaris->contains($comentari)) {
            $this->comentaris->removeElement($comentari);
            // set the owning side to null (unless already changed)
            if ($comentari->getUsuari() === $this) {
                $comentari->setUsuari(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Valoracio[]
     */
    public function getValoracions(): Collection
    {
        return $this->valoracions;
    }

    public function addValoracion(Valoracio $valoracion): self
    {
        if (!$this->valoracions->contains($valoracion)) {
            $this->valoracions[] = $valoracion;
            $valoracion->setUsuari($this);
        }

        return $this;
    }

    public function removeValoracion(Valoracio $valoracion): self
    {
        if ($this->valoracions->contains($valoracion)) {
            $this->valoracions->removeElement($valoracion);
            // set the owning side to null (unless already changed)
            if ($valoracion->getUsuari() === $this) {
                $valoracion->setUsuari(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection|Valoracio[]
     */
    public function getValoracionsFrom(): Collection
    {
        return $this->valoracions_from;
    }

    public function addValoracionsFrom(Valoracio $valoracionsFrom): self
    {
        if (!$this->valoracions_from->contains($valoracionsFrom)) {
            $this->valoracions_from[] = $valoracionsFrom;
            $valoracionsFrom->setUsuariDesti($this);
        }

        return $this;
    }

    public function removeValoracionsFrom(Valoracio $valoracionsFrom): self
    {
        if ($this->valoracions_from->contains($valoracionsFrom)) {
            $this->valoracions_from->removeElement($valoracionsFrom);
            // set the owning side to null (unless already changed)
            if ($valoracionsFrom->getUsuariDesti() === $this) {
                $valoracionsFrom->setUsuariDesti(null);
            }
        }

        return $this;
    }

    public function getImatge(): ?string
    {
        return $this->imatge;
    }

    public function setImatge(?string $imatge): self
    {
        $this->imatge = $imatge;

        return $this;
    }
}
